/* **********************************************************************
 * File: idpl.h
 * Desc: [I]magen [D]igital [P]rocessing [L]ibrary
 * Auto: Francisco Javier Olmedilla Torralba
 * Vers: 0.0
 * ********************************************************************** */
#ifndef IDP_H
#define IDP_H
#include <gtk/gtk.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/time.h>

#define LN2		0.6931471805599453094172321214581766  /* log_e 2 */
#define PI		3.14159265358979323846  /* pi */
#define INFINITO	65536
#define C_R 0.299       // Constantes para pasar una image RGB
#define C_G 0.587       // a otra en escala de grises.
#define C_B 0.114
#define WIDTH_LUT 30	// ancho de la imagen que representa el LUT
#define HEIGHT_LUT 10	// alto de la imagen q representa el LUT en la ventana del histograma
#define ROUND(x)        ((int)(x+0.5))  // Redondeo al entero mas cercano

#define TE_TARGET_HIST	"hist_acu"
#define TE_FLAGS_HIST	0
#define TE_ID_HIST	0

#define TE_TARGET_IMAGE	"image"
#define TE_FLAGS_IMAGE	0
#define TE_ID_IMAGE	1


#define TE_TARGET_GAUSS "gauss"
#define TE_FLAGS_GAUSS	0
#define TE_ID_GAUSS	2

/*
 * ESTRUCTURAS
 */
typedef struct
{
    guchar *buf;		// Niveles de gris de los pxeles
    int width; 			// Ancho de la image (pixeles)
    int height;			// Alto de la image
    int min;			// Minimo valor de nivel de gris
    int max;			// Mximo
    float bright;		// Brillo (media de los niveles de gris)
    float contrast;		// Contraste (desviacin tpica)
    float entropy;		// Entropa
    unsigned int *hist_abs;
    unsigned int *hist_acu;
    GdkRgbCmap *cmap;		// Look Up Table. Parece ser que cuando se llama a la funcin gdk_rgb_cmap_new () se crea una copia
    				// del array que se le pasa.
} image_t;

typedef struct
{
    image_t *image;
    int id;
} less_bits_t;

typedef struct 
{
    image_t *image;
    GtkWidget *statusbar;
} image_motion_t;

typedef struct
{
    image_t *image;
    int type;
    GtkWidget *statusbar;
} histogram_motion_t;

typedef struct 
{
    int *points;
    GtkWidget *spin_x;
    GtkWidget *spin_y;
    int	n;
    GdkPixmap *pixmap;
    GtkWidget *darea;
} lineal_t;

typedef struct
{
    image_t* image;
    double* gauss;
} gauss_t;

typedef struct
{
    int *v;
    int *v_d;
    int *v_s_d;
} cross_section_t;

/* para la interpolacion bilineal */
typedef struct
{
    int x, y;
} point_t;

typedef struct
{
    point_t A, B, C, D;
} corners_t;

typedef struct
{
    double A, B, C, D;	/* realmente con un "guchar" hubiera sido suficiente
			   pero ponemos "double" para los calculos en 
			   "I_BILINEAL" */
} corners_grays_t;

/* x, y valores enteros */
#define POINT(x,y)	(point_t){x,y}

/* x, y valores reales */
#define CORNERS(x,y)	(corners_t){\
				POINT((int)x, (int)y),\
				POINT((int)x+1, (int)y),\
				POINT((int)x, (int)y+1),\
				POINT((int)x+1, (int)y+1)}
				
/* x,y valores enteros */
#define GRAYS(im,x,y)		(im->cmap->colors[im->buf[im->width*y+x]]&0xff)
				
/* colores para las esquinar (imagen, esquinas) */
#define CORNERS_GRAYS(im,c)	(corners_grays_t){\
				(double)GRAYS(im, c.A.x, c.A.y),\
				(double)GRAYS(im, c.B.x, c.B.y),\
				(double)GRAYS(im, c.C.x, c.C.y),\
				(double)GRAYS(im, c.D.x, c.D.y)}
				
/* interpolacion bilineal (corners_gray, p, q) */
#define I_BILINEAL(cg,p,q)	(ROUND((cg.C + (cg.D-cg.C)*p + \
	    			(cg.A-cg.C)*q + \
				(cg.B+cg.C-cg.A-cg.D)*p*q)))
/*
 * PROTOTIPOS
 */

/* ------------------------------------------ PROCESAMIENTO DE IMAGENES --------------------------------------------------- */

guchar* 	idp_image_new_from_tfe_file 		(const gchar *file_name);
image_t*	idp_image_new 				(guchar *buf, int width, int height, guint *colors);
image_t*	idp_image_new_with_hist_abs			(guchar *buf, int width, int height, guint *colors, unsigned int *hist_abs);
image_t*	idp_image_new_from_file			(const gchar *file_name);
image_t*	idp_image_new_subimage			(image_t *image, int x, int y, int width, int height);
image_t*	idp_image_new_negative			(image_t *image);
image_t*	idp_image_new_lineal			(image_t *image, int *points);
image_t*	idp_image_new_bright_contrast		(image_t *image, float bright, float contrast);
image_t*	idp_image_new_equalized			(image_t *image);
image_t*	idp_image_new_histogram_acu		(image_t *image, unsigned int *hist_acu);
image_t*	idp_image_new_gauss			(int width, int height, double med, double dt, double n1, double n2, double** gauss);
image_t*	idp_image_new_less_samples		(image_t *image, int gx, int gy);
image_t*	idp_image_new_less_bits			(image_t *image, int nb);
image_t*	idp_image_new_difference		(image_t *img1, image_t* img2);
image_t*	idp_image_new_logarithm			(image_t *image);
image_t*	idp_image_new_logarithm_k		(image_t *image, double k);
image_t*	idp_image_new_logarithm_histogram_k	(image_t *image, double k);
image_t*	idp_image_new_add_gauss			(image_t *image, double *gauss);
image_t*	idp_image_new_exponential		(image_t *image);
image_t*	idp_image_copy				(image_t *image);
void		idp_image_free				(image_t *image);

/* ----------------------------------------- VISUALIZACION DE IMAGENES, HISTOGRAMAS ... ------------------------------------ */

void		idp_window_with_image			(image_t *image);
void		idp_window_with_histogram2		(image_t *image, int type);
void		idp_window_with_histogram		(unsigned int *histogram);
void		idp_window_with_lut			(guint *colors);		// NO USAR
void 		idp_window_with_image_and_changes 	(image_t *image, char *changes);
void 		idp_window_with_gauss			(image_t *image, double *gauss);
void		idp_window_with_cross_section 		(int *value, int *value_d, int *value_s_d, int n);

/* ---------------------------------------- CAPTACIN DE EVENTOS ----------------------------------------------------------- */

gboolean 	on_pixmap_expose_event	 		(GtkWidget *widget, GdkEventExpose *event, GdkPixmap *pixmap);
#define 	on_histogram_expose_event 		on_pixmap_expose_event
#define		on_changes_expose_event 		on_pixmap_expose_event
#define		on_lineal_expose_event			on_pixmap_expose_event
gboolean 	on_image_expose_event 			(GtkWidget *widget, GdkEventExpose *event, image_t *image);
gboolean 	on_lut_expose_event 			(GtkWidget *widget, GdkEventExpose *event, guchar *colors);
void		on_pixmap_destroyed			(GtkWidget *widget, GdkPixmap *pixmap);
#define		on_histogram_destroyed			on_pixmap_destroyed
#define		on_changes_destroyed			on_pixmap_destroyed
#define		on_lineal_destroyed			on_pixmap_destroyed
void 		on_image_destroyed 			(GtkWidget *widget, image_t *image);
void 		on_lut_destroyed 			(GtkWidget *widget, guchar *buf);
void 		on_gauss_destroyed 			(GtkWidget *widget, gauss_t* gg);
void		on_cross_section_destroyed 		(GtkWidget *widget, cross_section_t *cs);
void 		on_target_drag_data_received		(GtkWidget *widget, GdkDragContext *context, guint x, guint y, GtkSelectionData *data, guint info, guint time, gpointer udata);
void 		on_source_drag_data_get_histogram	(GtkWidget *widget, GdkDragContext *context, GtkSelectionData *selection_data, guint info, guint time, gpointer data);
void 		on_source_drag_data_get_image		(GtkWidget *widget, GdkDragContext *context, GtkSelectionData *selection_data, guint info, guint time, gpointer data);
void 		on_source_drag_data_get_gauss		(GtkWidget *widget, GdkDragContext *context, GtkSelectionData *selection_data, guint info, guint time, gpointer data);
void		on_image_motion_notify_event		(GtkWidget *widget, GdkEventMotion *event, image_motion_t* im);
void		on_histogram_motion_notify_event	(GtkWidget *widget, GdkEventMotion *event, histogram_motion_t* histogram);
void		on_prueba				(GtkWidget *widget, GtkAllocation *allocation, gpointer data);

    
/* --------------------------------- RUTIMAS DE APOYO PARA EL PROCESAMIENTO DE IMAGENES ------------------------------------- */

GdkRgbCmap*	create_gray_lut 			(void);
unsigned int*	create_absolute_histogram 		(image_t *image);
unsigned int*	create_acumulative_histogram 		(unsigned int *absolute);
void 		range_of_grays_levels 			(int *min, int *max, unsigned int *absolute);
float 		bright_of_image 			(image_t *image);
float		entropy_of_image			(image_t *image);
float		contrast_of_image			(image_t *image);
double*		normal 					(double m, double s, int n, double n1, double n2);
char*		pixels_changes				(image_t *image, float n);
GdkPoint 	*points_to_image_cross_section 		(int x1, int y1, int x2, int y2, int *n);
int*		value_to_image_cross_section 		(image_t *image, GdkPoint *points, int n);
int*		value_to_image_cross_section_smoothed 	(int *value, int n);
int*		value_to_image_cross_section_deriv 	(int *value, int n);
    
/* ------------------------------------------------------- MENU --------------------------------------------------------------- */

GtkWidget*	create_menu				(image_t *image);
gint		menu_popup				(GtkWidget *widget, GdkEventButton *event, gpointer data);
gint 		menu_negative 				(GtkWidget *widget, gpointer data);
gint 		menu_equalized 				(GtkWidget *widget, gpointer data);
gint 		menu_histogram 				(GtkWidget *widget, gpointer data);
gint 		menu_less_bits 				(GtkWidget *widget, gpointer data);
gint 		menu_less_samples			(GtkWidget *widget, gpointer data);
gint 		menu_gauss				(GtkWidget *widget, gpointer data);
gint 		menu_subimage				(GtkWidget *widget, gpointer data);
gint 		menu_bright_contrast			(GtkWidget *widget, gpointer data);
gint 		menu_lineal				(GtkWidget *widget, gpointer data);
gint 		menu_cross_section			(GtkWidget *widget, gpointer data);
gint 		menu_logarithm				(GtkWidget *widget, gpointer data);
gint 		menu_logarithm_k			(GtkWidget *widget, gpointer data);
gint 		menu_logarithm_histogram_k		(GtkWidget *widget, gpointer data);
gint 		menu_exponential			(GtkWidget *widget, gpointer data);
gint 		menu_exponential_k			(GtkWidget *widget, gpointer data);
gint 		menu_exponential_histogram_k		(GtkWidget *widget, gpointer data);

gint 		menu_vertical_mirror		(GtkWidget *widget, gpointer data);
gint 		menu_horizontal_mirror		(GtkWidget *widget, gpointer data);
gint 		menu_rotation_90		(GtkWidget *widget, gpointer data);
gint 		menu_rotation			(GtkWidget *widget, gpointer data);
gint 		menu_resize			(GtkWidget *widget, gpointer data);
/* ------------------------- VENTANAS DE DIALOGO PARA PEDIR PARMETROS NECESARIOS PARA UNA OPERACIN --------------------------- */

unsigned char	dialog_less_samples			(int* x, int* y);
unsigned char 	dialog_gauss 				(int* width, int* height, double* m, double* s, double* n1, double* n2);
unsigned char 	dialog_subimage 			(int* x, int* y, int* width, int* height);
unsigned char 	dialog_bright_contrast 			(float* bright, float* contrast);
unsigned char 	dialog_lineal	 			(int *points);
unsigned char 	dialog_cross_section 			(int* x1, int* y1, int* x2, int* y2);
unsigned char 	dialog_logarithm_k 			(double *k);
unsigned char 	dialog_rotation 			(int *degree, int *type, int *direct);
unsigned char 	dialog_resize 			(int *percent_x, int *percent_y, int *type);

/* ---------------------------------------------- RUTINAS DE APOYO PARA LOS DILOGOS ------------------------------------------- */

GtkWidget*	frame_two_spin_buttons 		(const char* str, int minx, int maxx, int miny, int maxy, GtkWidget **spin_x, GtkWidget **spin_y);
GtkWidget*	frame_gauss 			(const char* str , int width, int height,  GtkWidget **spint_w, GtkWidget  **spint_h, GtkWidget  **spint_m, GtkWidget **spint_s, GtkWidget **spint_n1, GtkWidget **spint_n2);

/* ------------------------------ RUTINAS DE APOYO PARA LA INTERFAZ DEL AJUSTE LINEAL POR TRAMOS ------------------------------- */

void		draw_lineal				(GtkWidget *darea, GdkPixmap *pixmap, int *points, int n);
void		on_spinx_lineal_value_changed		(GtkSpinButton *spin_x, gpointer data);
void		on_spiny_lineal_value_changed		(GtkSpinButton *spin_y, gpointer data);

/* ----------------------------------- RUTINAS PARA EL DIBUJADO DE LOS PERFILES ------------------------------------------------- */

void 		draw_cross_section 			(GdkPixmap *pixmap, int *value, int n);
void		draw_graph_cross_section		(GtkWidget *vbox, char *str, int *value, int n, int height);

/* xxxxxxxxxxxxxx TRANSFORMACIONES GEOMETRICAS xxxxxxxxxxxxxxx */
image_t 	*idp_image_new_horizontal_mirror 	(image_t* image);
image_t 	*idp_image_new_vertical_mirror 	(image_t* image);
image_t 	*idp_image_new_rotation_90 	(image_t* image);
image_t 	*idp_image_new_resize	 	(image_t*, int, int, char);
image_t 	*idp_image_new_rotation 	(image_t*, int, char, char);
#endif
